@@ -15,6 +15,7 @@
 #include "viewfactory.h"
 
 #include <avogadro/core/elements.h>
+#include <avogadro/core/variant.h>
 #include <avogadro/io/cjsonformat.h>
 #include <avogadro/io/cmlformat.h>
 #include <avogadro/io/fileformat.h>
@@ -55,6 +56,7 @@
 #include <QtGui/QCloseEvent>
 #include <QtGui/QDesktopServices>
 #include <QtGui/QKeySequence>
+#include <QtGui/QPalette>
 
 #include <QtNetwork/QNetworkAccessManager>
 #include <QtNetwork/QNetworkReply>
@@ -94,7 +96,7 @@
 #include <avogadro/vtk/vtkglwidget.h>
 #endif
 
-#ifdef Q_OS_MAC
+#if defined(Q_OS_MAC) && (QT_VERSION < QT_VERSION_CHECK(6, 0, 0))
 #include "qcocoamessagebox.h"
 #define MESSAGEBOX QCocoaMessageBox
 #else
@@ -453,8 +455,10 @@ void MainWindow::setupInterface()
 
   // Create the scene plugin model
   m_sceneTreeView->setAlternatingRowColors(true);
-  m_sceneTreeView->header()->stretchLastSection();
+  m_sceneTreeView->header()->setStretchLastSection(false);
   m_sceneTreeView->header()->setVisible(false);
+  if (m_sceneTreeView->header()->count() > 0)
+    m_sceneTreeView->header()->setSectionResizeMode(0, QHeaderView::Stretch);
   connect(m_sceneTreeView, &QAbstractItemView::activated, this,
           &MainWindow::sceneItemActivated);
   connect(m_sceneTreeView, &QAbstractItemView::clicked, this,
@@ -904,7 +908,7 @@ bool MainWindow::openFile(const QString& fileName, Io::FileFormat* reader)
   if (m_fileReadMolecule)
     m_fileReadMolecule->deleteLater();
   m_fileReadMolecule = new Molecule(this);
-  m_fileReadMolecule->setData("fileName", fileName.toLocal8Bit().data());
+  m_fileReadMolecule->setData("fileName", qPrintable(fileName));
   m_threadedReader->moveToThread(m_fileReadThread);
   m_threadedReader->setMolecule(m_fileReadMolecule);
   m_threadedReader->setFileName(fileName);
@@ -940,13 +944,42 @@ void MainWindow::backgroundReaderFinished()
     delete m_fileReadMolecule;
   } else if (m_threadedReader->success()) {
     if (!fileName.isEmpty()) {
-      m_fileReadMolecule->setData("fileName", fileName.toLocal8Bit().data());
+      m_fileReadMolecule->setData("fileName", qPrintable(fileName));
       m_recentFiles.prepend(fileName);
       updateRecentFiles();
     } else {
       m_fileReadMolecule->setData("fileName", Core::Variant());
     }
+
     setMolecule(m_fileReadMolecule);
+
+    // check if the modelView is set
+    if (m_fileReadMolecule->hasData("modelView")) {
+      MatrixX m = m_fileReadMolecule->data("modelView").value<MatrixX>();
+      // convert to an Affine3f for the camera
+      Eigen::Affine3f a;
+      a.matrix() = m.cast<float>();
+
+      if (auto* glWidget = qobject_cast<QtOpenGL::GLWidget*>(
+            m_multiViewWidget->activeWidget())) {
+        glWidget->renderer().camera().setModelView(a);
+        glWidget->requestUpdate();
+      }
+    }
+    // and the projection matrix
+    if (m_fileReadMolecule->hasData("projection")) {
+      MatrixX m = m_fileReadMolecule->data("projection").value<MatrixX>();
+      // convert to an Affine3f for the camera
+      Eigen::Affine3f a;
+      a.matrix() = m.cast<float>();
+
+      if (auto* glWidget = qobject_cast<QtOpenGL::GLWidget*>(
+            m_multiViewWidget->activeWidget())) {
+        glWidget->renderer().camera().setProjection(a);
+        glWidget->requestUpdate();
+      }
+    }
+
     statusBar()->showMessage(tr("Molecule loaded (%1 atoms, %2 bonds)")
                                .arg(m_molecule->atomCount())
                                .arg(m_molecule->bondCount()),
@@ -981,8 +1014,7 @@ bool MainWindow::backgroundWriterFinished()
     if (m_threadedWriter->success()) {
       statusBar()->showMessage(
         tr("Saved file %1", "%1 = filename").arg(fileName));
-      m_threadedWriter->molecule()->setData("fileName",
-                                            fileName.toLocal8Bit().data());
+      m_threadedWriter->molecule()->setData("fileName", qPrintable(fileName));
       markMoleculeClean();
       updateWindowTitle();
       success = true;
@@ -1168,6 +1200,11 @@ void MainWindow::viewActivated(QWidget* widget)
   if (auto* glWidget = qobject_cast<GLWidget*>(widget)) {
     bool firstRun = populatePluginModel(glWidget->sceneModel(), this);
     m_sceneTreeView->setModel(&glWidget->sceneModel());
+    // tweak the size of columns
+    m_sceneTreeView->header()->setSectionResizeMode(1, QHeaderView::Fixed);
+    m_sceneTreeView->header()->resizeSection(1, 40);
+    m_sceneTreeView->header()->setSectionResizeMode(0, QHeaderView::Stretch);
+
     populateTools(glWidget);
 
     foreach (ExtensionPlugin* extension, m_extensions) {
@@ -1379,6 +1416,21 @@ bool MainWindow::saveFile(bool async)
     return saveFileAs(async);
   }
 
+  // get the camera modelView and projection to save it
+  if (auto* glWidget =
+        qobject_cast<QtOpenGL::GLWidget*>(m_multiViewWidget->activeWidget())) {
+
+    auto affine = glWidget->renderer().camera().modelView();
+    MatrixX m = affine.matrix().cast<double>();
+    Core::Variant modelView(m);
+    mol->setData("modelView", modelView);
+
+    affine = glWidget->renderer().camera().projection();
+    m = affine.matrix().cast<double>();
+    Core::Variant projection(m);
+    mol->setData("projection", projection);
+  }
+
   if (!mol->hasData("fileName"))
     return saveFileAs(async);
 
@@ -1557,6 +1609,21 @@ bool MainWindow::saveFileAs(const QString& fileName, Io::FileFormat* writer,
     return false;
   }
 
+  // get the camera modelView and projection to save it
+  if (auto* glWidget =
+        qobject_cast<QtOpenGL::GLWidget*>(m_multiViewWidget->activeWidget())) {
+
+    auto affine = glWidget->renderer().camera().modelView();
+    MatrixX m = affine.matrix().cast<double>();
+    Core::Variant modelView(m);
+    mol->setData("modelView", modelView);
+
+    affine = glWidget->renderer().camera().projection();
+    m = affine.matrix().cast<double>();
+    Core::Variant projection(m);
+    mol->setData("projection", projection);
+  }
+
   // Prepare the background thread to write the selected file.
   m_threadedWriter->moveToThread(m_fileWriteThread);
   m_threadedWriter->setMolecule(mol);
@@ -1910,11 +1977,17 @@ void MainWindow::buildMenu()
   m_fileToolBar->addAction(action);
   connect(action, SIGNAL(triggered()), SLOT(saveFileAs()));
 
-  // Export
+  // Export action for menu
   QStringList exportPath = path;
   exportPath << tr("&Export");
   action = new QAction(tr("&Molecule…"), this);
   m_menuBuilder->addAction(exportPath, action, 110);
+#ifndef Q_OS_MAC
+  action->setIcon(QIcon::fromTheme("document-export"));
+#endif
+  connect(action, SIGNAL(triggered()), this, SLOT(exportFile()));
+  // Export action for toolbar with more clear name
+  action = new QAction(tr("Export Molecule…"), this);
   m_fileToolBar->addAction(action);
 #ifndef Q_OS_MAC
   action->setIcon(QIcon::fromTheme("document-export"));
@@ -2106,12 +2179,20 @@ void MainWindow::buildTools()
 {
   PluginManager* plugin = PluginManager::instance();
 
+  // determine if need dark mode or light mode icons
+  // e.g. https://www.qt.io/blog/dark-mode-on-windows-11-with-qt-6.5
+  const QPalette defaultPalette;
+  // is the text lighter than the window color?
+  bool darkMode = (defaultPalette.color(QPalette::WindowText).lightness() >
+                   defaultPalette.color(QPalette::Window).lightness());
+
   // Now the tool plugins need to be built/added.
   QList<ToolPluginFactory*> toolPluginFactories =
     plugin->pluginFactories<ToolPluginFactory>();
   foreach (ToolPluginFactory* factory, toolPluginFactories) {
     ToolPlugin* tool = factory->createInstance(QCoreApplication::instance());
     tool->setParent(this);
+    tool->setIcon(darkMode);
     if (tool)
       m_tools << tool;
   }
@@ -2466,7 +2547,7 @@ void MainWindow::registerToolCommand(QString command, QString description)
   if (!tool)
     return;
 
-  m_toolCommandMap.insert(command, tool);
+  m_toolCommandMap.insert(command, tool->name());
 }
 
 void MainWindow::registerExtensionCommand(QString command, QString description)
@@ -2528,12 +2609,21 @@ bool MainWindow::handleCommand(const QString& command,
     if (glWidget == nullptr)
       return false;
 
-    auto* tool = m_toolCommandMap.value(command);
+    QString toolName = m_toolCommandMap.value(command);
     auto* currentTool = glWidget->activeTool();
-    glWidget->setActiveTool(tool->objectName());
+
+    // find the requested tool
+    auto* tool = currentTool;
+    foreach (ToolPlugin* toolPlugin, glWidget->tools()) {
+      if (toolPlugin->name() == toolName) {
+        glWidget->setActiveTool(toolPlugin);
+        tool = toolPlugin;
+        break;
+      }
+    }
+
     bool result = tool->handleCommand(command, options);
     glWidget->setActiveTool(currentTool);
-
     return result;
   } else if (m_extensionCommandMap.contains(command)) {
     auto* extension = m_extensionCommandMap.value(command);
